package com.bakdata.conquery.sql.conversion.model.aggregator;

import com.bakdata.conquery.models.common.IRange;
import com.bakdata.conquery.models.common.Range;
import com.bakdata.conquery.models.datasets.Column;
import com.bakdata.conquery.models.datasets.concepts.filters.specific.CountFilter;
import com.bakdata.conquery.models.datasets.concepts.select.connector.specific.CountSelect;
import com.bakdata.conquery.sql.conversion.cqelement.concept.ConceptCteStep;
import com.bakdata.conquery.sql.conversion.cqelement.concept.FilterContext;
import com.bakdata.conquery.sql.conversion.model.SqlTables;
import com.bakdata.conquery.sql.conversion.model.filter.CountCondition;
import com.bakdata.conquery.sql.conversion.model.filter.SqlFilters;
import com.bakdata.conquery.sql.conversion.model.filter.WhereClauses;
import com.bakdata.conquery.sql.conversion.model.select.ExtractingSqlSelect;
import com.bakdata.conquery.sql.conversion.model.select.FieldWrapper;
import com.bakdata.conquery.sql.conversion.model.select.SelectContext;
import com.bakdata.conquery.sql.conversion.model.select.SqlSelects;
import lombok.Value;
import org.jooq.Field;
import org.jooq.impl.DSL;


@Value
public class CountSqlAggregator implements SqlAggregator {

	SqlSelects sqlSelects;
	WhereClauses whereClauses;

	private CountSqlAggregator(
			Column countColumn,
			CountType countType,
			String alias,
			SqlTables connectorTables,
			IRange<? extends Number, ?> filterValue
	) {
		ExtractingSqlSelect<?> rootSelect = new ExtractingSqlSelect<>(connectorTables.getRootTable(), countColumn.getName(), Object.class);

		Field<?> qualifiedRootSelect = rootSelect.qualify(connectorTables.getPredecessor(ConceptCteStep.AGGREGATION_SELECT)).select();
		Field<Integer> countField = countType == CountType.DISTINCT
									? DSL.countDistinct(qualifiedRootSelect)
									: DSL.count(qualifiedRootSelect);
		FieldWrapper<Integer> countGroupBy = new FieldWrapper<>(countField.as(alias), countColumn.getName());

		SqlSelects.SqlSelectsBuilder builder = SqlSelects.builder()
														 .preprocessingSelect(rootSelect)
														 .aggregationSelect(countGroupBy);

		String finalPredecessor = connectorTables.getPredecessor(ConceptCteStep.AGGREGATION_FILTER);
		if (filterValue == null) {
			ExtractingSqlSelect<Integer> finalSelect = countGroupBy.qualify(finalPredecessor);
			this.sqlSelects = builder.finalSelect(finalSelect).build();
			this.whereClauses = WhereClauses.empty();
		}
		else {
			this.sqlSelects = builder.build();
			Field<Integer> qualifiedCountSelect = countGroupBy.qualify(finalPredecessor).select();
			CountCondition countCondition = new CountCondition(qualifiedCountSelect, filterValue);
			this.whereClauses = WhereClauses.builder()
											.groupFilter(countCondition)
											.build();
		}
	}

	public static CountSqlAggregator create(CountSelect countSelect, SelectContext selectContext) {
		return new CountSqlAggregator(
				countSelect.getColumn(),
				CountType.fromBoolean(countSelect.isDistinct()),
				selectContext.getNameGenerator().selectName(countSelect),
				selectContext.getTables(),
				null
		);
	}

	public static CountSqlAggregator create(CountFilter countFilter, FilterContext<Range.LongRange> filterContext) {
		return new CountSqlAggregator(
				countFilter.getColumn(),
				CountType.fromBoolean(countFilter.isDistinct()),
				filterContext.getNameGenerator().selectName(countFilter),
				filterContext.getTables(),
				filterContext.getValue()
		);
	}

	@Override
	public SqlFilters getSqlFilters() {
		return new SqlFilters(this.sqlSelects, this.whereClauses);
	}

	public enum CountType {
		DEFAULT,
		DISTINCT;

		public static CountType fromBoolean(boolean value) {
			return value ? DISTINCT : DEFAULT;
		}
	}

}
